# Soya 3D
# Copyright (C) 2001-2002 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import Tkinter, dircache, os.path
import editobj, editobj.main, editobj.editor as editor
import soya, soya.math3d as math3d, soya.soya3d as soya3d, soya.model as model, soya.particle as particle
import soya.editor.material, soya.editor.world

import soya.laser as laser
import soya.ray   as ray

#
# Config and hacks for EditObj.
#

# Add new properties (rotation are properties !))

def _look_at_xyz(self, x, y, z):
  super(soya3d.GraphicElement, self).look_at_xyz(x, y, z)
  if hasattr(self, "_EventObj_stuff"): self._EventObj_stuff(self, "x", None, None)
soya3d.GraphicElement.look_at_xyz = _look_at_xyz

def _rot_lateral(self, angle):
  super(soya3d.GraphicElement, self).rotate_lateral(angle)
  if hasattr(self, "_EventObj_stuff"): self._EventObj_stuff(self, "x", None, None)
soya3d.GraphicElement.rotate_lateral = _rot_lateral

def _rot_vertical(self, angle):
  super(soya3d.GraphicElement, self).rotate_vertical(angle)
  if hasattr(self, "_EventObj_stuff"): self._EventObj_stuff(self, "x", None, None)
soya3d.GraphicElement.rotate_vertical = _rot_vertical

def _rot_incline(self, angle):
  super(soya3d.GraphicElement, self).rotate_incline(angle)
  if hasattr(self, "_EventObj_stuff"): self._EventObj_stuff(self, "x", None, None)
soya3d.GraphicElement.rotate_incline = _rot_incline

soya3d.GraphicElement.rot_lateral  = property(None, lambda self, angle: self.rotate_lateral (angle))
soya3d.GraphicElement.rot_vertical = property(None, lambda self, angle: self.rotate_vertical(angle))
soya3d.GraphicElement.rot_incline  = property(None, lambda self, angle: self.rotate_incline (angle))

def _set_xyz(self, x, y, z): # For eventobj
  self.x = x
  self.y = y
  self.z = z
soya3d.GraphicElement.set_xyz = _set_xyz

model.Face.items = property(lambda self: self.vertices, None)
#particle.System.items = property(lambda self: self.particles, None)
particle.System.items = property(lambda self: [self.generator], None)

# Register properties' editors
class TextureCoordEditor(editor.WithButtonEditor):
  def __init__(self, master, obj, attr):
    editor.WithButtonEditor.__init__(self, master, obj, attr, editor.FloatEditor, "...")
    
  def button_click(self, event = None):
    if self.obj.face.material.tex_filename: TextureCoordDialog(self.obj)

_texture_coord_dialog = None

class TextureCoordDialog(Tkinter.Toplevel):
  def __init__(self, vertex):
    Tkinter.Toplevel.__init__(self)
    
    self.clicked = 0
    
    self.label = Tkinter.Label(self, bd = 0)
    self.label.pack()
    self.label.bind("<Button-1>"      , self.motion)
    self.label.bind("<Button1-Motion>", self.motion)
    
    self.set_vertex(vertex)
    
    global _texture_coord_dialog
    _texture_coord_dialog = self
    
  def destroy(self):
    Tkinter.Toplevel.destroy(self)
    
    global _texture_coord_dialog
    if _texture_coord_dialog is self: _texture_coord_dialog = None
    
  def set_vertex(self, vertex):
    self.vertex = vertex
    
    import Image, ImageTk
    self.image = ImageTk.PhotoImage(Image.open(model.Image(vertex.face.material.tex_filename)))
    self.label.configure(image = self.image)
    
  def motion(self, event):
    self.vertex.tex_x = float(event.x) / self.image.width ()
    self.vertex.tex_y = float(event.y) / self.image.height()

class XTextureCoordEditor(editor.Editor, Tkinter.Frame):
  require_right_menu = 0
  
  def __init__(self, master, obj, attr):
    editor.Editor.__init__(self, master, obj, attr)
    
    Tkinter.Frame.__init__(self, master)
    self.columnconfigure(0, weight = 1)
    
    if obj.face.material and obj.face.material.tex_filename:
      try:
        import Image, ImageTk
        
        self.image = Image.open(os.path.join(model.Image.PATH, obj.face.material.tex_filename))
        
        self.image_width = self.image_height = 0
        
        self.label = Tkinter.Label(self, bd = 0, width = 500)
        self.label.grid(row = 0, col = 0, sticky = "EWN")
        self.label.bind("<Configure>"     , self.label_conf)
        self.label.bind("<Button-1>"      , self.motion)
        self.label.bind("<Button1-Motion>", self.motion)
      except ImportError: # PIL is not installed !
        self.label = None
        
    self.internal_editor = editor.FloatEditor(self, obj, attr)
    self.internal_editor.grid(row = 1, col = 0, sticky = "EWS")
    self.cancel = self.master.cancel
    
  def label_conf(self, event = None):
    import Image, ImageTk
    
    if self.image_width != self.label.winfo_width():
      self.image_width  = self.label.winfo_width()
      self.image_height = (self.image.size[1] / float(self.image.size[0])) * self.image_width
      
      self.imagetk = ImageTk.PhotoImage(self.image.resize((self.image_width, self.image_height)))
      
      self.label.configure(image = self.imagetk)
      
  def get_value(self): return self.internal_editor.get_value()
  def set_value(self, value): self.internal_editor.set_value(value)
  def update(self): self.internal_editor.update()
  
  def motion(self, event):
    self.obj.tex_x = max(0.0, min(1.0, float(event.x) / self.image_width ))
    self.obj.tex_y = max(0.0, min(1.0, float(event.y) / self.image_height))
    
editor.register_attr("face"             , None)
editor.register_attr("normal"           , None)
editor.register_attr("vertices"         , None)
editor.register_attr("name"             , editor.StringEditor)
editor.register_attr("filename"         , editor.StringEditor)
editor.register_attr("x"                , editor.FloatEditor)
editor.register_attr("y"                , editor.FloatEditor)
editor.register_attr("z"                , editor.FloatEditor)
editor.register_attr("scale_x"          , editor.FloatEditor)
editor.register_attr("scale_y"          , editor.FloatEditor)
editor.register_attr("scale_z"          , editor.FloatEditor)
editor.register_attr("front"            , editor.FloatEditor)
editor.register_attr("back"             , editor.FloatEditor)
editor.register_attr("fov"              , editor.FloatEditor)
editor.register_attr("rot_lateral"      , editor.FloatEditor)
editor.register_attr("rot_vertical"     , editor.FloatEditor)
editor.register_attr("rot_incline"      , editor.FloatEditor)
editor.register_attr("red"              , editor.FloatEditor)
editor.register_attr("green"            , editor.FloatEditor)
editor.register_attr("blue"             , editor.FloatEditor)
editor.register_attr("alpha"            , editor.FloatEditor)
editor.register_attr("shininess"        , editor.FloatEditor)
editor.register_attr("point_size"       , editor.FloatEditor)
editor.register_attr("initial_speed"    , editor.FloatEditor)
editor.register_attr("exponent"         , editor.FloatEditor)
editor.register_attr("angle"            , editor.FloatEditor)
editor.register_attr("directional"      , editor.BoolEditor)
editor.register_attr("constant"         , editor.FloatEditor)
editor.register_attr("linear"           , editor.FloatEditor)
editor.register_attr("quadratic"        , editor.FloatEditor)
editor.register_attr("additive_blending", editor.BoolEditor)
editor.register_attr("separate_specular", editor.BoolEditor)
editor.register_attr("wireframed"       , editor.BoolEditor)
editor.register_attr("visible"          , editor.BoolEditor)
editor.register_attr("lit"              , editor.BoolEditor)
editor.register_attr("static_lit"       , editor.BoolEditor)
editor.register_attr("smooth_lit"       , editor.BoolEditor)
editor.register_attr("double_sided"     , editor.BoolEditor)
editor.register_attr("solid"            , editor.BoolEditor)
editor.register_attr("bound_atm"        , editor.BoolEditor)
editor.register_attr("camera_go_through", editor.BoolEditor)
editor.register_attr("infinite"         , editor.BoolEditor)
editor.register_attr("use_clip_plane"   , editor.BoolEditor)
editor.register_attr("static"           , editor.BoolEditor)
editor.register_attr("static_only"      , editor.BoolEditor)
editor.register_attr("top_level"        , editor.BoolEditor)
editor.register_attr("reflect"          , editor.BoolEditor)
editor.register_attr("acceleration"     , editor.SubEditor)
editor.register_attr("endpoint"         , editor.SubEditor)

def _all_parent(item):
  if isinstance(item, soya3d.World):
    inner_worlds = item.recursive()
    inner_worlds.append(item)
    return filter(lambda item: isinstance(item, soya3d.World) and (not item.name.startswith("_")) and (not item in inner_worlds), item.get_root().recursive())
  else:
    return filter(lambda item: isinstance(item, soya3d.World) and (not item.name.startswith("_")), item.get_root().recursive())
  
editor.register_attr("parent"           , editor.LambdaListEditor(_all_parent))
editor.register_attr("tex_filename"     , editor.LambdaListEditor(lambda obj: dircache.listdir(model.Image.PATH)    + [None]))

class MaterialEditor(editor.WithButtonEditor):
  INTERNAL_EDITOR_CLASS = editor.LambdaListEditor(lambda obj: filter(lambda filename: not (filename and filename.startswith("_")), model.Material.availables()), lambda filename: filename and model.Material.get(filename))
  
  def __init__(self, master, obj, attr):
    editor.WithButtonEditor.__init__(self, master, obj, attr, self.INTERNAL_EDITOR_CLASS, "...")
    
  def button_click(self, event = None):
    edit(getattr(self.obj, self.attr))
    
class ShapeEditor(editor.WithButtonEditor):
  INTERNAL_EDITOR_CLASS = editor.LambdaListEditor(lambda obj: filter(lambda filename: not (filename and filename.startswith("_")), model.Shape.availables()), lambda filename: filename and model.Shape.get(filename))
  
  def __init__(self, master, obj, attr):
    editor.WithButtonEditor.__init__(self, master, obj, attr, self.INTERNAL_EDITOR_CLASS, "...")
    
  def button_click(self, event = None):
    edit(getattr(self.obj, self.attr).to_world())
    
    #shape = getattr(self.obj, self.attr)
    #world = soya3d.World()
    #shape.to_faces(world)
    #edit(world)
    
    #edit(soya3d.World.get(getattr(self.obj, self.attr).filename))
    
class WorldEditor(editor.WithButtonEditor):
  INTERNAL_EDITOR_CLASS = editor.LambdaListEditor(lambda obj: filter(lambda filename: not (filename and filename.startswith("_")), soya3d.World.availables()), lambda filename: filename and soya3d.World.get(filename))
  
  def __init__(self, master, obj, attr):
    editor.WithButtonEditor.__init__(self, master, obj, attr, self.INTERNAL_EDITOR_CLASS, "...")
    
  def button_click(self, event = None):
    edit(getattr(self.obj, self.attr))
    
editor.register_attr("shape"            , ShapeEditor)
editor.register_attr("material"         , MaterialEditor)
editor.register_attr("beyond"           , WorldEditor)

editor.register_attr("image_path"       , editor.DirnameEditor)
editor.register_attr("material_path"    , editor.DirnameEditor)
editor.register_attr("shape_path"       , editor.DirnameEditor)
editor.register_attr("world_path"       , editor.DirnameEditor)

editor.register_attr("tex_x"            , XTextureCoordEditor)
editor.register_attr("tex_y"            , editor.FloatEditor)

editor.register_attr("reverse_vertices" , editor.CallableEditor)


def _reverse_vertices(self): self.vertices.reverse()

model.Face.reverse_vertices = _reverse_vertices
model.Face.__members__ = ["reverse_vertices"]

def ImmatureVertex():
  vertex = model.Vertex()
  vertex.immature = 1
  return vertex

_last_face = None

def _Gone(vertices):
  global _last_face
  
  f = model.Face(None, vertices)
  if _last_face:
    f.double_sided = _last_face.double_sided
    f.material     = _last_face.material
    f.smooth_lit = _last_face.smooth_lit
    f.lit = _last_face.lit
    f.solid = _last_face.solid
    f.static_lit = _last_face.static_lit
  _last_face = f
  return f

model.Triangle = lambda : _Gone([ImmatureVertex(), ImmatureVertex(), ImmatureVertex()])
model.Quad     = lambda : _Gone([ImmatureVertex(), ImmatureVertex(), ImmatureVertex(), ImmatureVertex()])

items_for_list = editor.config.available_items
#if not items_for_list.has_key(str(soya3d.World)):
items_for_list[str(soya3d.World)] = ["model.Triangle()", "model.Quad()", "soya3d.Volume()", "soya3d.World()", "soya3d.Camera()", "soya3d.Light()", "soya3d.Portal()", "particle.Fountain()", "laser.Laser()", "ray.Ray()"]
if not items_for_list.has_key(str(model .Face )): items_for_list[str(model .Face )] = ["model.Vertex()"]

values_for_attr = editor.config.available_attrs
if not values_for_attr.has_key("color"       ): values_for_attr["color"       ] = ["None", "(1.0, 1.0, 1.0, 1.0)", "(0.0, 0.0, 0.0, 1.0)"]
if not values_for_attr.has_key("diffuse"     ): values_for_attr["diffuse"     ] = ["None", "(1.0, 1.0, 1.0, 1.0)", "(0.0, 0.0, 0.0, 1.0)"]
if not values_for_attr.has_key("specular"    ): values_for_attr["specular"    ] = ["None", "(1.0, 1.0, 1.0, 1.0)", "(0.0, 0.0, 0.0, 1.0)"]
if not values_for_attr.has_key("shapify_args"): values_for_attr["shapify_args"] = ["None", """("tree", {"min_node_content":0, "min_node_radius":5.0, "min_node_distance":0.0})""", """("morph", {})"""]

editobj.EVAL_ENV.update({
  "soya"     : soya,
  "math3d"   : math3d,
  "soya3d"   : soya3d,
  "model"    : model,
  "particle" : particle,
  "laser"    : laser,
  "ray"      : ray,
  })

# Better Tk look
_tk = Tkinter.Tk(className = 'EditObj')
_tk.withdraw()
_tk.option_add("*bd", 1)
_tk.option_add("*List.background", "#FFFFFF")
_tk.option_add("*Text.background", "#FFFFFF")
_tk.option_add("*Text.relief", "flat")
_tk.option_add("*Entry.background", "#FFFFFF")
_tk.option_add("*Entry.relief", "flat")


CURRENT = None

def edit_material(self, window):
  if not soya.inited: soya.init("Soya Editor")
  
  ed = soya.editor.material.MaterialEditor(self, window)
  
  def on_activate(event = None):
    global CURRENT
    
    if CURRENT: CURRENT.deactivate()
    ed.activate()
    CURRENT = ed
    
  window.bind("<FocusIn>" , on_activate)

def edit_world(self, window):
  if not soya.inited: soya.init("Soya Editor")
  
  ed = soya.editor.world.WorldEditor(self, window)
  
  def on_activate(event = None):
    global CURRENT
    
    if CURRENT: CURRENT.deactivate()
    ed.activate()
    CURRENT = ed
    
  window.bind("<FocusIn>" , on_activate)

model.Material.__edit__ = edit_material
soya3d.World  .__edit__ = edit_world


def _subedit(self, window):
  CURRENT.select_handles_for(self)
  
def _subedit_vertex(self, window):
  CURRENT.select_handles_for(self)
  if _texture_coord_dialog: _texture_coord_dialog.set_vertex(self)
  
def _subedit_face(self, window):
  for vertex in self:
    CURRENT.select_handles_for(vertex)
    
def _editchildren(self, visible):
  if (not CURRENT) or not hasattr(CURRENT, "children_edited"): return
  CURRENT.children_edited(self, visible)
  
soya3d.GraphicElement.__subedit__      = _subedit
model.Vertex         .__subedit__      = _subedit_vertex
model.Face           .__subedit__      = _subedit_face
soya3d.World         .__editchildren__ = _editchildren
soya3d.Morph         .__editchildren__ = _editchildren

edit = editobj.edit
