# export blender model to soya

# this script make a Python script to rebuild the model in Soya editor and do not run directly 
# the editor cause Blender is not running Python 2.2 (needed by Soya)
#
# HOW TO EXPORT A BLENDER MODEL :
# - run Blender and load your model
# - delete useless objects like camera, lights, armature that can crash this script
# - load the script
# - change the following filename to where you want to save this script result

import os
filename = os.path.expanduser("~/last_model.py")

# - run the script (Alt-p). It will create a Python script with the given filename. Run this new
# script to have your model imported in Soya editor
# - before running the created script, you may want to change the texture names to the Soya materials
# names
#
# WARNING : there are some hack on this script since Blender python module is very few documented...
#   (for example for vertices color)

import Blender

def pointbymatrix(p, m):
  return [p[0] * m[0][0] + p[1] * m[1][0] + p[2] * m[2][0] + m[3][0],
          p[0] * m[0][1] + p[1] * m[1][1] + p[2] * m[2][1] + m[3][1],
          p[0] * m[0][2] + p[1] * m[1][2] + p[2] * m[2][2] + m[3][2]]

def export_to_soya():
  code = ""
  materials = []
  
  objs = Blender.Object.Get()
  print objs

  for obj in objs:

    # take only NMesh

    if(obj.data.block_type == "NMesh"):
      nmesh = Blender.NMesh.GetRaw(obj.data.name)

      matrix = [[obj.mat[0][0], obj.mat[0][1], obj.mat[0][2], obj.mat[0][3]],
                [obj.mat[1][0], obj.mat[1][1], obj.mat[1][2], obj.mat[1][3]],
                [obj.mat[2][0], obj.mat[2][1], obj.mat[2][2], obj.mat[2][3]],
                [obj.mat[3][0], obj.mat[3][1], obj.mat[3][2], obj.mat[3][3]]]

      for face in nmesh.faces:

        code = code + "f = soya.model.Face(root_world)\n"

        # face option
        if(face.smooth != 0):
          code = code + "f.smooth_lit = 1\n"
        #if(face.col != None):
        #if(face.twoside != 0):
        #  code = code + "f.double_sided = 1\n"

        # material
        if(face.image != None):
          m_name = ''
          for char in face.image.name:
            if(char == '.'): break
            m_name = m_name + char
          code = code + "f.material = " + m_name + "\n"
          if not m_name in materials: materials.append(m_name)
            
        # vertices
        index = 0
        for vertex in face.v:
          # vertex coordinates
          co = pointbymatrix(vertex.co, matrix)
          code = code + "v = soya.model.Vertex(root_world, " + str(co[0]) + ", " + str(co[1]) + ", " + str(co[2]) + ")\n"
          # vertex color
          if(face.col != None and len(face.col) > 0):
            color = face.col[index]
            code = code + "v.color = (" + str(color.r / 255.0) + ", " + str(color.g / 255.0) + ", " + str(color.b / 255.0) + ", " + str(color.a / 255.0) + ")\n"
          # vertex point normal
          # vertex.no[0]
          # vertex.no[1]
          # vertex.no[2]
          # vertex texture coordinates
          if(len(face.uv) > 0):
          #if(face.image):
            uv = face.uv[index]
            code = code + "v.tex_x = " + str(uv[0]) + "\nv.tex_y = " + str(1.0 - uv[1]) + "\n"
            # vertex.uvco[0]
            # vertex.uvco[1]
          code = code + "f.append(v)\n"
          index = index + 1

  full_code = """
# This file has been generated by Soya

import Tkinter, soya, soya.model, soya.soya3d, soya.editor

root_world = soya.soya3d.World()

# You may want to change these lines...
soya.model.Image.PATH    = '/home/jiba/python/slune/images'
soya.model.Material.PATH = '/home/jiba/python/slune/materials'
soya.soya3d.World.PATH   = '/home/jiba/python/slune/worlds'
soya.model.Shape.PATH    = '/home/jiba/python/slune/shapes'
"""

  for material in materials:
    full_code = full_code + material + " = soya.model.Material.get('" + material +"')\n"

  return full_code + "\n\n" + code + """
# This is to compensate cause upper axis is Z in blender, Y in Soya
root_world.rotate_lateral(-90.0)

soya.editor.edit(root_world)
Tkinter.mainloop()"""



file = open(filename, "w")
file.write(export_to_soya())
file.close()
print filename, "wrote\n"
